{
  This file was automatically generated on 2012-12-21.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletDualRelay;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_DUAL_RELAY_FUNCTION_SET_STATE = 1;
  BRICKLET_DUAL_RELAY_FUNCTION_GET_STATE = 2;
  BRICKLET_DUAL_RELAY_FUNCTION_SET_MONOFLOP = 3;
  BRICKLET_DUAL_RELAY_FUNCTION_GET_MONOFLOP = 4;
  BRICKLET_DUAL_RELAY_FUNCTION_GET_IDENTITY = 255;
  BRICKLET_DUAL_RELAY_CALLBACK_MONOFLOP_DONE = 5;

type
  TArray0To2OfUInt8 = array [0..2] of byte;

  TBrickletDualRelayNotifyMonoflopDone = procedure(sender: TObject; const relay: byte; const state: boolean) of object;

  /// <summary>
  ///  Device for controlling two relays
  /// </summary>
  TBrickletDualRelay = class(TDevice)
  private
    monoflopDoneCallback: TBrickletDualRelayNotifyMonoflopDone;
  protected
    procedure CallbackWrapperMonoflopDone(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid__: string; ipcon_: TIPConnection);

    /// <summary>
    ///  Sets the state of the relays, *true* means on and *false* means off. 
    ///  For example: (true, false) turns relay 1 on and relay 2 off.
    ///  
    ///  If you just want to set one of the relays and don't know the current state
    ///  of the other relay, you can get the state with <see cref="BrickletDualRelay.TBrickletDualRelay.GetState"/>.
    ///  
    ///  Running monoflop timers will be overwritten if this function is called.
    ///  
    ///  The default value is (*false*, *false*).
    /// </summary>
    procedure SetState(const relay1: boolean; const relay2: boolean); virtual;

    /// <summary>
    ///  Returns the state of the relays, *true* means on and *false* means off.
    /// </summary>
    procedure GetState(out relay1: boolean; out relay2: boolean); virtual;

    /// <summary>
    ///  The first parameter can be 1 or 2 (relay 1 or relay 2). The second parameter 
    ///  is the desired state of the relay (*true* means on and *false* means off).
    ///  The third parameter indicates the time (in ms) that the relay should hold 
    ///  the state.
    ///  
    ///  If this function is called with the parameters (1, true, 1500):
    ///  Relay 1 will turn on and in 1.5s it will turn off again.
    ///  
    ///  A monoflop can be used as a failsafe mechanism. For example: Lets assume you 
    ///  have a RS485 bus and a Dual Relay Bricklet connected to one of the slave 
    ///  stacks. You can now call this function every second, with a time parameter
    ///  of two seconds. The relay will be on all the time. If now the RS485 
    ///  connection is lost, the relay will turn off in at most two seconds.
    ///  
    ///  .. versionadded:: 1.1.1~(Plugin)
    /// </summary>
    procedure SetMonoflop(const relay: byte; const state: boolean; const time: longword); virtual;

    /// <summary>
    ///  Returns (for the given relay) the current state and the time as set by 
    ///  <see cref="BrickletDualRelay.TBrickletDualRelay.SetMonoflop"/> as well as the remaining time until the state flips.
    ///  
    ///  If the timer is not running currently, the remaining time will be returned
    ///  as 0.
    ///  
    ///  .. versionadded:: 1.1.1~(Plugin)
    /// </summary>
    procedure GetMonoflop(const relay: byte; out state: boolean; out time: longword; out timeRemaining: longword); virtual;

    /// <summary>
    ///  Returns the UID, the UID where the Bricklet is connected to, 
    ///  the position, the hardware and firmware version as well as the
    ///  device identifier.
    ///  
    ///  The position can be 'a', 'b', 'c' or 'd'.
    ///  
    ///  The device identifiers are:
    ///  
    ///  <code>
    ///   "Device Identifier", "Device"
    ///  
    ///   "11", "Brick DC"
    ///   "12", "Brick Debug"
    ///   "13", "Brick Master 2.0"
    ///   "14", "Brick Servo"
    ///   "15", "Brick Stepper"
    ///   "16", "Brick IMU"
    ///   "", ""
    ///   "21", "Bricklet Ambient Light"
    ///   "22", "Bricklet Breakout"
    ///   "23", "Bricklet Current12"
    ///   "24", "Bricklet Current25"
    ///   "25", "Bricklet Distance IR"
    ///   "26", "Bricklet Dual Relay"
    ///   "27", "Bricklet Humidity"
    ///   "28", "Bricklet IO-16"
    ///   "29", "Bricklet IO-4"
    ///   "210", "Bricklet Joystick"
    ///   "211", "Bricklet LCD 16x2"
    ///   "212", "Bricklet LCD 20x4"
    ///   "213", "Bricklet Linear Poti"
    ///   "214", "Bricklet Piezo Buzzer"
    ///   "215", "Bricklet Rotary Poti"
    ///   "216", "Bricklet Temperature"
    ///   "217", "Bricklet Temperature IR"
    ///   "218", "Bricklet Voltage"
    ///   "219", "Bricklet Analog In"
    ///   "220", "Bricklet Analog Out"
    ///   "221", "Bricklet Barometer"
    ///   "222", "Bricklet GPS"
    ///   "223", "Bricklet Industrial Digital In 4"
    ///   "224", "Bricklet Industrial Digital Out 4"
    ///   "225", "Bricklet Industrial Quad Relay"
    ///   "226", "Bricklet PTC"
    ///   "227", "Bricklet Voltage/Current"
    ///  </code>
    ///  
    ///  .. versionadded:: 2.0.0~(Plugin)
    /// </summary>
    procedure GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TArray0To2OfUInt8; out firmwareVersion: TArray0To2OfUInt8; out deviceIdentifier: word); virtual;

    /// <summary>
    ///  This callback is triggered whenever a monoflop timer reaches 0. The 
    ///  parameter contain the relay (1 or 2) and the current state of the relay 
    ///  (the state after the monoflop).
    ///  
    ///  .. versionadded:: 1.1.1~(Plugin)
    /// </summary>
    property OnMonoflopDone: TBrickletDualRelayNotifyMonoflopDone read monoflopDoneCallback write monoflopDoneCallback;
  end;

implementation

constructor TBrickletDualRelay.Create(const uid__: string; ipcon_: TIPConnection);
begin
  inherited Create(uid__, ipcon_);
  apiVersion[0] := 1;
  apiVersion[1] := 0;
  apiVersion[2] := 1;

  responseExpected[BRICKLET_DUAL_RELAY_FUNCTION_SET_STATE] := RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICKLET_DUAL_RELAY_FUNCTION_GET_STATE] := RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_DUAL_RELAY_FUNCTION_SET_MONOFLOP] := RESPONSE_EXPECTED_FALSE;
  responseExpected[BRICKLET_DUAL_RELAY_FUNCTION_GET_MONOFLOP] := RESPONSE_EXPECTED_ALWAYS_TRUE;
  responseExpected[BRICKLET_DUAL_RELAY_CALLBACK_MONOFLOP_DONE] := RESPONSE_EXPECTED_ALWAYS_FALSE;
  responseExpected[BRICKLET_DUAL_RELAY_FUNCTION_GET_IDENTITY] := RESPONSE_EXPECTED_ALWAYS_TRUE;

  callbackWrappers[BRICKLET_DUAL_RELAY_CALLBACK_MONOFLOP_DONE] := {$ifdef FPC}@{$endif}CallbackWrapperMonoflopDone;
end;

procedure TBrickletDualRelay.SetState(const relay1: boolean; const relay2: boolean);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreatePacket(self, BRICKLET_DUAL_RELAY_FUNCTION_SET_STATE, 10);
  LEConvertBooleanTo(relay1, 8, request);
  LEConvertBooleanTo(relay2, 9, request);
  SendRequest(request);
end;

procedure TBrickletDualRelay.GetState(out relay1: boolean; out relay2: boolean);
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreatePacket(self, BRICKLET_DUAL_RELAY_FUNCTION_GET_STATE, 8);
  response := SendRequest(request);
  relay1 := LEConvertBooleanFrom(8, response);
  relay2 := LEConvertBooleanFrom(9, response);
end;

procedure TBrickletDualRelay.SetMonoflop(const relay: byte; const state: boolean; const time: longword);
var request: TByteArray;
begin
  request := (ipcon as TIPConnection).CreatePacket(self, BRICKLET_DUAL_RELAY_FUNCTION_SET_MONOFLOP, 14);
  LEConvertUInt8To(relay, 8, request);
  LEConvertBooleanTo(state, 9, request);
  LEConvertUInt32To(time, 10, request);
  SendRequest(request);
end;

procedure TBrickletDualRelay.GetMonoflop(const relay: byte; out state: boolean; out time: longword; out timeRemaining: longword);
var request, response: TByteArray;
begin
  request := (ipcon as TIPConnection).CreatePacket(self, BRICKLET_DUAL_RELAY_FUNCTION_GET_MONOFLOP, 9);
  LEConvertUInt8To(relay, 8, request);
  response := SendRequest(request);
  state := LEConvertBooleanFrom(8, response);
  time := LEConvertUInt32From(9, response);
  timeRemaining := LEConvertUInt32From(13, response);
end;

procedure TBrickletDualRelay.GetIdentity(out uid: string; out connectedUid: string; out position: char; out hardwareVersion: TArray0To2OfUInt8; out firmwareVersion: TArray0To2OfUInt8; out deviceIdentifier: word);
var request, response: TByteArray; i: longint;
begin
  request := (ipcon as TIPConnection).CreatePacket(self, BRICKLET_DUAL_RELAY_FUNCTION_GET_IDENTITY, 8);
  response := SendRequest(request);
  uid := LEConvertStringFrom(8, 8, response);
  connectedUid := LEConvertStringFrom(16, 8, response);
  position := LEConvertCharFrom(24, response);
  for i := 0 to 2 do hardwareVersion[i] := LEConvertUInt8From(25 + (i * 1), response);
  for i := 0 to 2 do firmwareVersion[i] := LEConvertUInt8From(28 + (i * 1), response);
  deviceIdentifier := LEConvertUInt16From(31, response);
end;

procedure TBrickletDualRelay.CallbackWrapperMonoflopDone(const packet: TByteArray);
var relay: byte; state: boolean;
begin
  if (Assigned(monoflopDoneCallback)) then begin
    relay := LEConvertUInt8From(8, packet);
    state := LEConvertBooleanFrom(9, packet);
    monoflopDoneCallback(self, relay, state);
  end;
end;

end.
